"""WorkToy - Core
This package provides core types."""
#  MIT Licence
#  Copyright (c) 2023 Asger Jon Vistisen
from __future__ import annotations

from ._coretypes import Map, Keys, Values, Items, Bases, Type
from ._coretypes import Function, Method, WrapperDescriptor
from ._coretypes import WrapperMethod, BuiltinFunction, Functional
from ._coretypes import FunctionTuple, FunctionList
from ._coretypes import ARGS, KWARGS, RESULT, CALL
from ._math import PI, factorial, EXP1
