"""WorkToy - Descriptors
Implementation of descriptors as descriptors."""
#  MIT Licence
#  Copyright (c) 2023 Asger Jon Vistisen
from __future__ import annotations

from ._field import Field
from ._attribute import AbstractAttribute
from ._int_attribute import IntAttribute
from ._float_attribute import FloatAttribute
from ._str_attribute import StrAttribute
from ._flag import Flag
