"""WorkToy - MetaWork
This module provides the metaclass and namespace classes for the remaining
package."""
#  MIT Licence
#  Copyright (c) 2023 Asger Jon Vistisen
from __future__ import annotations

import worktoy.core as core
import worktoy.worktoyclass as worktoyclass
import worktoy.metaclass as metaclass
import worktoy.descriptors as descriptors
import worktoy.sym as sym
import worktoy.waitaminute as waitaminute
