"""WorkToy - MetaClass
Implementation of metaclass support."""
#  MIT Licence
#  Copyright (c) 2023 Asger Jon Vistisen
from __future__ import annotations

from ._meta_metaclass import MetaMetaClass
from ._abstract_metaclass import AbstractMetaClass
from ._meta_namespace import MetaNameSpace
from ._abstract_namespace import AbstractNameSpace